import React, { useState } from "react";
import { uploadVideo } from "../../utils/upload";

const VideoUploader = ({ setIds }) => {
  const [selectedVideos, setSelectedVideos] = useState([]);
  const [uploading, setUploading] = useState(false);

  const handleFileChange = async (event) => {
    const files = event.target.files;

    if (files.length > 0) {
      const newVideos = Array.from(files).map((file) => {
        return {
          file,
          videoUrl: URL.createObjectURL(file),
          uploadStatus: "pending", // Initial status is pending
        };
      });

      setSelectedVideos((prevVideos) => [...prevVideos, ...newVideos]);

      // Upload videos
      for (const newVideo of newVideos) {
        await uploadVideoAsync(newVideo);
      }
    }
  };

  const uploadVideoAsync = async (video) => {
    try {
      setUploading(true);

      const uploadTimeout = 30000; // Set timeout to 30 seconds
      const uploadPromise = uploadVideo(video.file);
      const uploadResult = await Promise.race([
        uploadPromise,
        new Promise((_, reject) =>
          setTimeout(() => reject(new Error("Upload timeout")), uploadTimeout)
        ),
      ]);

      if (typeof uploadResult === "number") {
        // Upload successful
        video.uploadStatus = "success";
        // Pass the uploaded integer to the parent component
        setIds((prevIds) => [...prevIds, uploadResult]);
      } else {
        // Upload failed
        video.uploadStatus = "fail";
      }
      console.log(`Video ${video.file.name} upload result:`, uploadResult);
    } catch (error) {
      console.error(`Failed to upload video ${video.file.name}`, error);
      video.uploadStatus = "fail";
    } finally {
      setUploading(false);
      // Update the state to trigger a re-render
      setSelectedVideos((prevVideos) => [...prevVideos]);
    }
  };

  return (
    <div>
      <div style={{ display: "flex", flexWrap: "wrap" }}>
        {selectedVideos.map((video, index) => (
          <div key={index} style={{ margin: "5px" }} className="relative">
            <video
              controls
              src={video.videoUrl}
              alt={`视频 ${index + 1}`}
              style={{
                width: "100px",
                height: "100px",
                objectFit: "cover",
                borderRadius: "8px",
              }}
            />
            {video.uploadStatus === "success" && (
              <p className="absolute top-0 left-0 text-green-400 font-bold bg-black/50 flex w-full h-full items-center justify-center rounded-lg">
                上传成功
              </p>
            )}
            {video.uploadStatus === "fail" && (
              <p className="absolute top-0 left-0 text-red-400 font-bold bg-black/50 flex w-full h-full items-center justify-center rounded-lg">
                上传失败
              </p>
            )}
          </div>
        ))}
        <label
          htmlFor="fileInput"
          style={{
            width: "100px",
            height: "100px",
            margin: "5px",
            background: "#9ca3af",
            display: "flex",
            alignItems: "center",
            justifyContent: "center",
            cursor: "pointer",
            position: "relative",
            borderRadius: "8px",
          }}
        >
          <span className="text-white">
            {uploading ? "正在上传..." : "添加视频"}
          </span>
          <input
            id="fileInput"
            type="file"
            multiple
            onChange={handleFileChange}
            style={{
              opacity: 0,
              position: "absolute",
              top: 0,
              left: 0,
              width: "100%",
              height: "100%",
              cursor: "pointer",
            }}
          />
        </label>
      </div>
    </div>
  );
};

export default VideoUploader;
